# How to run JavaScript and TypeScript examples and benchmarks

## Necessary software

To run the examples, it is necessary to install [Node.js](https://nodejs.org)
version 16 or higher.  Furthermore, provided examples need some Node.js packages
to work correctly.  Those packages can be installed by executing the following
command inside the OptalCP installation directory:

```sh
npm install
```

The command above installs the package locally into the directory `<OptalCP`
Installation>/node_modules/`. In particular, it installs support for TypeScript.

## Calling `optalcp` executable

OptalCP offers JavaScript and TypeScript APIs for Node.js (TypeScript
must be compiled into JavaScript before execution). The API allows the creation
of optimization models, the solve itself is done by executable `optalcp`
(from the `bin` directory of the installation).

Therefore, when running JavaScript benchmarks or examples, it is necessary to
specify where `optalcp` executable is. There are three ways:

### Option 1: `OPTALCP_SOLVER` environment variable

This is the preferred way. The variable should contain the whole path to the
`optalcp` executable (including the file name). For example, for Linux or Mac:

```sh
export OPTALCP_SOLVER=path/to/optalcp/bin/optalcp
node jobshop/jobshop.js jobshop/data/la17
```

The environment variable can be set once for all for example in `~/.profile`,
`~/.bashrc`, `~/.zshrc` or a similar file.

### Option 2: Command line parameter `--solverPath`

Another option is to specify the location of `optalcp` as a command line parameter:

```sh
node jobshop/jobshop.js --solverPath path/to/optalcp/bin/optalcp jobshop/data/la17
```

All delivered benchmarks and examples support `--solverPath` parameter.
To support `--solverPath` parameter in your code you can use
the function `parseArgs` or `parseSubset` of the class `CP.Parameters`.

### Option 3: `SolverPath` solver parameter

The last option is to use the function `setSolverPath` of the class `CP.Parameters`:

```js
import * as CP from '@scheduleopt/optalcp';

...
let params = new CP.Parameters;
params.setSolverPath("path/to/optalcp/optalcp");
CP.solve(model, params);
```

## Changing parameters

All examples and benchmarks allow changing solver parameters on the command line.
Even if a parameter is set in the source code, it can be changed on the command line.

For example:

```sh
node jobshop/jobshop.js jobshop/data/la24 --searchType fds --nbWorkers 4 --noOverlapPropagationLevel 4 --worker1.searchType lns --worker1.noOverlapPropagationLevel 2
```

The command line above specifies that the jobshop benchmark should be solved
using 4 workers (threads). First, all threads are configured to use
Failure-Directed Search (FDS) and maximum propagation for `noOverlap` (level 4).
Then search type of worker 1 is changed to Large-Neighborhood Search (LNS) and
similarly propagation of `noOverlap`.

TODO:1 Link to the documentation of supported parameters.

## Development in TypeScript

All provided examples come in two versions: TypeScript (original source code)
and JavaScript (compiled from TypeScript). This way the JavaScript code can be
run directly using Node.js. However, if you change the TypeScript code (or start
your own file), then it is necessary to compile TypeScript into JavaScript. It
can be done by issuing the following command in OptalCP installation directory:

```sh
npx tsc
```

TODO:1 Add a link to the documentation here.
